program test_set_max_speed

  ! Libraries:
  use contour_531, only: convert_to_ind, null_polyline
  use gpc_f, only: shp_read_pol, polygon
  use jumble, only: get_command_arg_dyn, assert, deg_to_rad
  use shapelib, only: shpfileobject, shpclose
  use shapelib_03, only: shp_open_03, shp_get_info_03, dbf_read_attribute_03

  use config_m, only: config
  use cont_list_m, only: create_cont_list, close_cont_list
  use derived_types, only: eddy, shpc_slice_handler, ssh_contour
  use input_ssh_m, only: input_ssh, corner
  use read_eddy_m, only: read_eddy
  use set_max_speed_m, only: set_max_speed
  use shpc_close_m, only: shpc_close
  use shpc_create_m, only: shpc_create
  use shpc_open_m, only: shpc_open
  use write_eddy_m, only: write_eddy

  implicit none

  logical cyclone
  logical periodic ! grid is periodic in longitude
  integer nlon, nlat, d, eddy_index, n_cont, i, ishape
  real, allocatable:: ssh(:, :) ! (nlon, nlat) sea-surface height, in m
  real, allocatable:: u(:, :), v(:, :) ! (nlon, nlat) wind, in m s-1
  type(eddy) e
  real step(2) ! in rad
  TYPE(shpc_slice_handler) hshpc
  character(len = :), allocatable:: path

  ! Window around the extremum:

  integer llc(2) ! indices in global grid of lower left corner
  integer urc(2) ! indices in global grid of upper right corner

  real corner_window(2) ! longitude and latitude of the window around
  ! the extremum, in rad

  integer, parameter:: n_max_cont = 31

  type(ssh_contour) cont_list(n_max_cont)
  ! Contour list. The outermost contour is element with subscript
  ! n_cont. The contours are in monotonic order of ssh.

  TYPE(shpfileobject) hshp
  type(polygon) p

  !----------------------------------------------------------------

  print *, "test_set_max_speed: cyclone = ? "
  read *, cyclone
  call config
  call input_ssh(step, periodic, ssh, u, v)
  print *, "test_set_max_speed: Reading from shapefiles..."
  call get_command_arg_dyn(3, path, "Required 3rd argument: SHPC-directory")
  call shpc_open(hshpc, path, cyclone, slice = 0, pszaccess = "rb")
  call read_eddy(e, d, eddy_index, hshpc, ishape = 0)
  CALL shpc_close(hshpc)
  if (.not. periodic) nlon = size(ssh, 1)
  nlat = size(ssh, 2)

  if (e%out_cont%closed) then
     ! Restrict the field to the outermost contour:

     llc = floor(convert_to_ind(minval(e%out_cont%points, dim = 2), &
          corner, step))

     urc = ceiling(convert_to_ind(maxval(e%out_cont%points, dim = 2), &
          corner, step))

     ! Should have no effect except because of roundup error:
     urc(2) = min(urc(2), nlat)
     if (.not. periodic) urc(1) = min(urc(1), nlon)

     corner_window = corner + (llc - 1) * step
     e%extr%coord_proj = nint((e%extr%coord - corner) / step) + 1

     call get_command_arg_dyn(4, path, &
          "Required 4th argument: cont-list-shapefile")
     call shp_open_03(hshp, path, "rb")
     call shp_get_info_03(hshp, n_cont)
     call assert(1 <= n_cont, n_cont <= n_max_cont - 1, &
          "test_set_max_speed n_cont")

     do i = 1, n_cont
        ishape = i - 1
        call shp_read_pol(p, hshp, ishape)

        if (p%nparts == 0) then
           cont_list(i)%polyline = null_polyline()
        else
           cont_list(i)%polyline = p%part(1)
           cont_list(i)%polyline%points &
                = cont_list(i)%polyline%points * deg_to_rad
        end if

        call dbf_read_attribute_03(cont_list(i)%ssh, hshp, ifield = 0, &
             ishape = ishape)
     end do

     call shpclose(hshp)
     call create_cont_list
     call set_max_speed(e%speed_cont, e%max_speed, cont_list, n_cont, &
          e%extr%coord, ssh(llc(1):urc(1), llc(2):urc(2)), &
          u(llc(1):urc(1), llc(2):urc(2)), v(llc(1):urc(1), llc(2):urc(2)), &
          corner_window, step)
     call close_cont_list
     call shpc_create(hshpc, shpc_dir = "SHPC", cyclone = cyclone, slice = 0, &
          grid_lon_lat = .true.)
     call write_eddy(e, hshpc, d, eddy_index)
     write(hshpc%unit, fmt = *) 0
     CALL shpc_close(hshpc)
     print *, 'test_set_max_speed: Created "SHPC" and "cont_list".'
  else
     print *, "No outermost contour found."
  end if

end program test_set_max_speed
